﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using FlatDataSourceBasic.WPF.Data;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;
using Infragistics.Olap;

namespace FlatDataSourceBasic.WPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            FlatDataSource flatDataSource = new FlatDataSource();

            // listening for new item is added in order to expand the root node
            // when working with flat data this is the most easiest way to do that 
            flatDataSource.Columns.CollectionChanging += this.AreaItems_CollectionChanging;
            flatDataSource.Rows.CollectionChanging += this.AreaItems_CollectionChanging;

            this.SetInitialItems(flatDataSource);
            this.SetHierarchies(flatDataSource);
            this.SetMetadata(flatDataSource);

            flatDataSource.ItemsSource = ProductDataGenerator.Generate();
            xamPivotGrid.DataSource = flatDataSource;
            xamPivotDataSelector.DataSource = flatDataSource;
        }

        private void AreaItems_CollectionChanging(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (var newItem in e.NewItems)
                {
                    IFilterViewModel filterViewModel = newItem as IFilterViewModel;
                    if (filterViewModel != null && filterViewModel.FilterMembers != null)
                    {
                        // expand the root level members
                        foreach (IFilterMember filterMember in filterViewModel.FilterMembers)
                        {
                            filterMember.IsExpanded = true;
                        }
                    }
                }
            }
        }

        private void SetInitialItems(FlatDataSource flatDataSource)
        {
            flatDataSource.Rows = DataSourceBase.GenerateInitialItems("[ProductCategory].[Product Categories]");
            flatDataSource.Columns = DataSourceBase.GenerateInitialItems("[Region].[Destributors by Regions]");
            flatDataSource.Measures = DataSourceBase.GenerateInitialItems("Cost");
        }

        private void SetHierarchies(FlatDataSource flatDataSource)
        {
            // ##### [ProductCategory].[Product Categories] Hierarchy #####
            // suppose we don't have the generic type parameter (dynamic type)
            HierarchyDescriptor productHierarchy =
                new HierarchyDescriptor
                {
                    SourcePropertyName = "ProductCategory",
                    HierarchyName = "Product Categories"
                };

            // use this method to add the root level
            productHierarchy.AddLevel("All Products", "All Products");

            HierarchyLevelDescriptor categoriesLevel = new HierarchyLevelDescriptor();
            categoriesLevel.LevelExpressionPath = "ProductCategory";
            categoriesLevel.LevelName = "Categories";

            HierarchyLevelDescriptor brandsLevel = new HierarchyLevelDescriptor();
            brandsLevel.LevelExpressionPath = "Brand";
            brandsLevel.LevelName = "Brands";

            HierarchyLevelDescriptor productsLevel = new HierarchyLevelDescriptor();
            productsLevel.LevelExpressionPath = "ProductName";
            productsLevel.LevelName = "Products";

            // add the levels to the hierarchy
            productHierarchy.LevelDescriptors.Add(categoriesLevel);
            productHierarchy.LevelDescriptors.Add(brandsLevel);
            productHierarchy.LevelDescriptors.Add(productsLevel);

            flatDataSource.AddHierarchyDescriptor(productHierarchy);

            // ##### [Region].[Destributors by Regions] Hierarchy #####
            HierarchyDescriptor regionHierarchy =
                new HierarchyDescriptor()
                {
                    SourcePropertyName = "Region",
                    HierarchyName = "Destributors by Regions"
                };

            // use this method to add the root level
            regionHierarchy.AddLevel("All Regions", "All Regions");

            HierarchyLevelDescriptor regionLevel = new HierarchyLevelDescriptor();
            regionLevel.LevelExpressionPath = "Region";
            regionLevel.LevelName = "Regions";

            HierarchyLevelDescriptor distributorsLevel = new HierarchyLevelDescriptor();
            distributorsLevel.LevelExpressionPath = "Distributor";
            distributorsLevel.LevelName = "Distributors";

            // add the levels to the hierarchy
            regionHierarchy.LevelDescriptors.Add(regionLevel);
            regionHierarchy.LevelDescriptors.Add(distributorsLevel);

            flatDataSource.AddHierarchyDescriptor(regionHierarchy);
        }

        private void SetMetadata(FlatDataSource flatDataSource)
        {
            CubeMetadata cubeMetadata =
                new CubeMetadata { DataTypeFullName = typeof(ProductData).FullName, DisplayName = "Products Data" };
            flatDataSource.CubesSettings.Add(cubeMetadata);

            DimensionMetadata costMetadata = new DimensionMetadata();
            costMetadata.SourcePropertyName = "Cost";
            costMetadata.DimensionType = DimensionType.Measure;

            cubeMetadata.DimensionSettings.Add(costMetadata);
        }
    }
}
